# cdplibs/sim/modsel/modfuncs.py                                    DMQ 08/19/04
"""Functions for a generic model selector.  Use this module as a pattern to
integrate your own analog simulator.
"""
import os
from os import listdir
from os.path import isdir, isfile, basename, join, abspath

def expandMods(modList):
    """Convert a compact model list into a simple, fully-expanded
model list, suitable for input to a dumb simulator: one model per
line, with full pathnames.  Use only the first model of any with
identical filenames.

For a single directory, list all model files.
>>> for p in expandMods([allnom]):
...     print p
./cdplibs/sim/modsamp/cs65/spectre/allnom/nmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/npn.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnp.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnps.scs   

To over-ride a single model, put the custom model first.  Section names,
if present, appear at the end of the line.
>>> for p in expandMods([pnps,allnom]):
...     print p
./cdplibs/sim/modsamp/cs65/spectre/custom/pnps.scs   slow
./cdplibs/sim/modsamp/cs65/spectre/allnom/nmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/npn.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnp.scs   

A custom model at the end of the list will not replace one earlier in
the sequence.  The second pnps is ignored.
>>> for p in expandMods([allnom, pnps]):
...     print p
./cdplibs/sim/modsamp/cs65/spectre/allnom/nmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/npn.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnp.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnps.scs   

Here is a typical model list with two over-ridden models.  The dead
link should generate an error message.
>>> for p in sampModList:
...     print p
./cdplibs/sim/modsamp/cs65/spectre/custom/pnps.scs;slow
./cdplibs/sim/modsamp/gpdk/nmos.scs
./cdplibs/sim/modsamp/gpdk/deadlink/
./cdplibs/sim/modsamp/cs65/spectre/allnom/

>>> for p in expandMods(sampModList):
...     print p
*** WARNING *** Not a file or dir:
'./cdplibs/sim/modsamp/gpdk/deadlink/'
./cdplibs/sim/modsamp/cs65/spectre/custom/pnps.scs   slow
./cdplibs/sim/modsamp/gpdk/nmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/npn.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pmos.scs   
./cdplibs/sim/modsamp/cs65/spectre/allnom/pnp.scs   ###xxx
"""
    mlist = []; plist = []  # model and path lists
    for row in modList:     # process rows top to bottom
        row = row.strip()   # strip off space on ends
        rs = row.split(';') # separate section name
        if len(rs) == 1:    # no section name
            row = rs[0]; section = ''
        elif len(rs) == 2:  # normal section name
            row = rs[0]; section = rs[1]
        else:
            print "***WARNING*** Bad Syntax:\n", rs
        
        if isdir(row):
            rlist = listdir(row)
            for mf in rlist:  # assume all items are simple model files
                rmf = join(row, mf)
                if not isfile(rmf):
                    print "*** WARNING *** Not a file:\n", rmf
                    break
                # more tests here (section name, etc.)
                if mf not in mlist: # skip later models with same name
                    mlist.append(mf)
                    plist.append(rmf + '   ' + section)
                        # use form required by your simulator.
        elif isfile(row):
            mf = basename(row)
            if mf not in mlist:   # ditto for simple files
                mlist.append(mf)
                plist.append(row + '   ' + section)
        else:
            print "*** WARNING *** Not a file or dir:\n", "'" + row + "'"
            
    return plist

